"""
Script for adding dataset mixture tasks to Seqio registry
Used for two things:
    - Mixture between URMP baseline and custom datasets
    - Mixture between all custom datasets generated by a single audio plugin (or baseline dataset)
"""

import functools
import seqio

# URMP 25 50 75 Mixture
def add_urmp_baseline_experiment_mixture(name):
    """
    Add mixture between UMRP baseline and a partition specified in parameter in 25%, 50%, and 75%
    :param name: The partition to mix
    """
    seqio.MixtureRegistry.add(
      f"{name}_25_notes_ties_vb1_train",
      [("urmp_baseline_notes_ties_vb1_train", 3), (f"{name}_notes_ties_vb1_train", 1)]
    )

    seqio.MixtureRegistry.add(
      f"{name}_50_notes_ties_vb1_train",
      [("urmp_baseline_notes_ties_vb1_train", 1), (f"{name}_notes_ties_vb1_train", 1)]
    )

    seqio.MixtureRegistry.add(
      f"{name}_75_notes_ties_vb1_train",
      [("urmp_baseline_notes_ties_vb1_train", 1), (f"{name}_notes_ties_vb1_train", 3)]
    )

    urmp_eval_names = [f'{name}_25_notes_ties_vb1_eval',
                       f'{name}_25_notes_ties_vb1_validation',
                       f'{name}_50_notes_ties_vb1_eval',
                       f'{name}_50_notes_ties_vb1_validation',
                       f'{name}_75_notes_ties_vb1_eval',
                       f'{name}_75_notes_ties_vb1_validation',]

    for urmp_eval in urmp_eval_names:
        seqio.MixtureRegistry.add(
            urmp_eval,
            ['urmp_baseline_notes_ties_vb1_validation'],
            default_rate=1)


# Add mixtures for SerumFX and Kontakt
add_urmp_baseline_experiment_mixture('urmp_e1_serumfx')
add_urmp_baseline_experiment_mixture('urmp_e2_kontakt')

# BASELINE MIXTURE
DATASET_NAMES = ['maestro', 'guitarset', 'musicnet', 'urmp']
EXP_NAMES = ['baseline', 'e1_rc20', 'e1_serumfx', 'e2_kontakt']
MIXING_TEMPERATURE = 10 / 3

for exp in EXP_NAMES:
    mixture_dataset_names = []
    mixture_eval_names = []
    for dataset in DATASET_NAMES:
        mixture_dataset_names.append(f"{dataset}_{exp}_notes_ties_vb1_train")
        mixture_eval_names.append(f"{dataset}_{exp}_notes_ties_vb1_eval")

    seqio.MixtureRegistry.add(f'full_{exp}_notes_ties_vb1_train',
        mixture_dataset_names,
        default_rate=functools.partial(
            seqio.mixing_rate_num_examples,
            temperature=MIXING_TEMPERATURE))
    seqio.MixtureRegistry.add(f'full_{exp}_notes_ties_vb1_eval',
        mixture_eval_names,
        default_rate=functools.partial(
            seqio.mixing_rate_num_examples,
            temperature=MIXING_TEMPERATURE))