import note_seq

class SequenceTools:
    """
    Contains tools for conversion of <note_seq.NoteSequence>
    """    

    @staticmethod
    def note_sequence_to_string(ns: note_seq.NoteSequence) -> str:
        """ Serializes note sequence to string

        :param ns: NoteSequence object
        """
        return ns.SerializeToString()
    
    @staticmethod
    def string_to_note_sequence(string: str) -> note_seq.NoteSequence:
        """ Deserialize string to note sequence
        
        :param string: Serialized string of a NoteSequence object
        """
        return note_seq.NoteSequence.FromString(string)

    @staticmethod
    def note_sequence_to_midi(ns: note_seq.NoteSequence, output_file: str) -> None:
        """ Convert note sequence to midi file
        
        :param ns: NoteSequence object to convert
        :param output_file: Output path of MIDI file to convert to
        """
        # TODO: Drop events?
        note_seq.midi_io.note_sequence_to_midi_file(ns, output_file, drop_events_n_seconds_after_last_note=None)

    @staticmethod
    def midi_to_note_sequence(input_file: str) -> note_seq.NoteSequence:
        """ Convert MIDI file to note sequence 
        
        :param input_file: Input path of MIDI file to convert from
        """
        return note_seq.midi_io.midi_file_to_note_sequence(input_file)

    @staticmethod
    def midi_to_string(input_file: str) -> str:
        """ Convert MIDI file to serialized note sequence string
        
        :param input_file: Input path of MIDI file to convert from
        """
        return SequenceTools.note_sequence_to_string(SequenceTools.midi_to_note_sequence(input_file))

    @staticmethod
    def string_to_midi(string: str, output_file: str) -> None:
        """ Convert serialized note sequence string to MIDI

        :param string: Serialized note sequence string
        :param output_file: Output path of MIDI file to convert to
        """
        return SequenceTools.note_sequence_to_midi(SequenceTools.string_to_note_sequence(string), output_file)


if __name__ == '__main__':
    ns = SequenceTools.midi_to_note_sequence("../EFGABCD.mid")
    notes = sorted(ns.notes,
                   key=lambda note: (note.is_drum, note.program, note.pitch))
    programs = []
    for note in notes:
        if note.program not in programs:
            programs.append(note.program)
    print(programs)