import tensorflow as tf
import pathlib

from Dataset.sequence_tools import SequenceTools

class TFRecordUtils:
    @staticmethod
    def serialize_example(data):
        """Serializes an example with provided data."""
        feature = {
            "id": TFRecordUtils._bytes_feature([TFRecordUtils._string_to_bytes(data["id"])]),
            "audio": TFRecordUtils._bytes_feature([TFRecordUtils._file_to_bytes(data["audio"])]),
            "sequence": TFRecordUtils._bytes_feature([SequenceTools.midi_to_string(data["sequence"])]),
            #"instrument_sequences": TFRecordUtils._bytes_feature([SequenceTools.midi_to_string(x) for x in data["instrument_sequences"]]),
            #"program_num": TFRecordUtils._int64_feature(data["program_num"]),
            #"inst_names": TFRecordUtils._bytes_feature([TFRecordUtils._string_to_bytes(str(x)) for x in data["program_num"]])
        }
        example_proto = tf.train.Example(features=tf.train.Features(feature=feature))
        return example_proto.SerializeToString()

    @staticmethod
    def _string_to_bytes(string) -> bytes:
        """Converts a string into bytes."""
        return string.encode('utf-8')

    @staticmethod
    def _file_to_bytes(input_file) -> bytes:
        """Converts a file in input path to bytes."""
        if pathlib.Path(input_file).is_file():  # Guard for file
            with open(input_file, 'rb') as file:
                return file.read()
        else:
            return b''

    @staticmethod
    def _bytes_feature(value):
        """Returns a bytes_list from a string / byte."""
        if isinstance(value, type(tf.constant(0))):
            value = value.numpy() # BytesList won't unpack a string from an EagerTensor.
        return tf.train.Feature(bytes_list=tf.train.BytesList(value=value))

    @staticmethod
    def _float_feature(value):
        """Returns a float_list from a float / double."""
        return tf.train.Feature(float_list=tf.train.FloatList(value=value))

    @staticmethod
    def _int64_feature(value):
        """Returns an int64_list from a bool / enum / int / uint."""
        return tf.train.Feature(int64_list=tf.train.Int64List(value=value))
