import os
import re
import shutil
import csv

"""
Functions to pack the datasets from original (downloaded) form, into a systematic manner that makes sense when rendering. 
"""


def pack_musicnet(wd=".", pack_folder="."):
    for file in os.listdir(wd):
        # Check if file is relevant
        if re.match("[0-9]{4}(.*)(.wav|.mid|.csv)$", file):
            file_nr = re.split('[._]', file)[0]
            track_dir = f"{wd}/{pack_folder}/{file_nr}"
            if not os.path.exists(track_dir):
                os.mkdir(track_dir)
            shutil.move(f"{wd}/{file}", f"{track_dir}/{file}")


def pack_guitarset(wd=".", pack_folder="."):
    for file in os.listdir(wd):
        # Check if the file is relevant
        if re.match("[0-9]{2}(.*)(.wav|.jams)", file):
            track_name = file.replace("_hex", "").split(".")[0]
            file_nr = re.split("[_-]", file)[1][-1]
            if file_nr in ['0', '1']:
                ds_type = "train"
            else:
                ds_type = "validation"
            track_folder = f"{wd}/{pack_folder}/{ds_type}/{track_name}"
            if not os.path.exists(track_folder):
                os.mkdir(track_folder)
            shutil.move(f"{wd}/{file}", f"{track_folder}/{file}")


def pack_maestro(wd=".", output_dir="."):
    midi_dict = _create_maestro_meta_dict(wd, 4)
    audio_dict = _create_maestro_meta_dict(wd, 5)
    for directory in os.listdir(f"{wd}"):
        if re.match("^[0-9]{4}$", directory):
            for file in os.listdir(f"{wd}/{directory}"):
                file_folder = file.split(".")[0]
                if file.endswith(".mid") or file.endswith(".midi"):
                    split = midi_dict[file]["split"]
                else:
                    split = audio_dict[file]["split"]
                file_folder = f"{output_dir}/{split}/{file_folder}"
                if not os.path.exists(file_folder):
                    os.mkdir(file_folder)
                shutil.move(f"{wd}/{directory}/{file}", f"{file_folder}/{file}")


def _create_maestro_meta_dict(wd, key_index=0):
    return_dict = {}
    with open(f'{wd}/maestro-v3.0.0.csv') as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        csv_reader = [i for i in csv_reader]
        headers = csv_reader[0]
        for row in csv_reader[1:]:
            attributes = {}
            for i in range(len(row)):
                if i != key_index:
                    attributes[headers[i]] = row[i]
            uuid = row[key_index].split("/")[-1]
            return_dict[uuid] = attributes
    return return_dict


if __name__ == "__main__":
    pack_maestro("../Datasets/maestro-v3.0.0.0/", "../Datasets/maestro-v3.0.0")

