from json import loads
from paramiko import SSHClient, AutoAddPolicy
from config import *
import pickle
import numpy as np
import matplotlib.pyplot as plt
import seqio
from tabulate import tabulate


def create_score_graphs(pickles, val_folder, x_ticks=None, measurements=(
"Onset F1", "Onset recall", "Onset precision", "Onset + offset F1", "Frame F1", "Frame Recall", "Frame Precision"), header=None):
    files = []
    for obj in pickles:
        infile = open(obj, "rb")
        new_dict = pickle.load(infile)
        files.append(new_dict[list(new_dict.keys())[0]])

    scores = [[] for _ in measurements]
    yerr = [[[], []] for _ in measurements]
    # Retrieve scores for each and merge
    for j, file in enumerate(files):
        bottom_top_score = get_validation_files_by_score(pickles[j], val_folder, 1,
                                                         measurements=measurements)
        for i in range(len(measurements)):
            scores[i].append(file[measurements[i]])
            # Get the full measurements from file
            # Place in [] and add to yerr for score
            yerr[i][0].append(file[measurements[i]] - bottom_top_score[measurements[i]][0][1])
            yerr[i][1].append(bottom_top_score[measurements[i]][1][1] - file[measurements[i]])

    # Create graph
    bar_width = (1 / len(measurements)) - 0.015

    # The x position of bars
    rows = [np.arange(len(scores[0]))]
    for i in range(1, len(scores)):
        rows.append([x + bar_width for x in rows[i - 1]])

    for i in range(len(rows)):
        #plt.bar(rows[i], scores[i], width=bar_width, label=measurements[i], yerr=yerr[i], linestyle="dotted")
        plt.bar(rows[i], scores[i], width=0.01, label=measurements[i], linestyle="dotted")

    if x_ticks:
        plt.xticks([r + (bar_width * len(measurements) / 2 - bar_width / 2) for r in range(len(rows[0]))], x_ticks)
    plt.ylabel('Prediction Accuracy')
    plt.title(header)
    plt.legend()

    # Show graphic
    plt.show()


def get_validation_files_by_score(eval_pickle, remote_folder, lim=None, measurements=(
"Onset F1", "Onset recall", "Onset precision", "Onset + offset F1", "Frame F1", "Frame Recall", "Frame Precision")):
    infile = open(eval_pickle, "rb")
    scores = pickle.load(infile)
    scores = scores[list(scores.keys())[0]]
    # SSH To server and get files in order
    ssh_client = SSHClient()
    ssh_client.set_missing_host_key_policy(AutoAddPolicy())
    ssh_client.connect(hostname=SSH_HOST, username=SSH_USER, password=SSH_PASSWORD)
    sftp_client = ssh_client.open_sftp()
    filenames = sftp_client.listdir(remote_folder)
    # Map score to files
    file_score_mappings = {}
    for m in measurements:
        measurement_scores = scores[f"{m} (hist)"]
        measurement_scores = measurement_scores.values
        mapping = []
        if lim:
            keep_indices = get_bottom_n_indices(measurement_scores, lim) + get_top_n_indices(measurement_scores, lim)
            for i in range(len(filenames)):
                if i in keep_indices:
                    mapping.append((filenames[i], measurement_scores[i]))
                    mapping.sort(key=lambda x: -x[1])
        else:
            for i in range(len(filenames)):
                mapping.append((filenames[i], measurement_scores[i]))
                mapping.sort(key=lambda x: -x[1])
        file_score_mappings[m] = mapping
    return file_score_mappings


def get_top_n_indices(col, n):
    index_value_pairs = [(i, col[i]) for i in range(len(col))]
    index_value_pairs = sorted(index_value_pairs, key=lambda x: -x[1])
    top_indices = []
    for pair in index_value_pairs[:n]:
        top_indices.append(pair[0])
    return top_indices


def get_bottom_n_indices(col, n):
    index_value_pairs = [(i, col[i]) for i in range(len(col))]
    index_value_pairs = sorted(index_value_pairs, key=lambda x: x[1])
    top_indices = []
    for pair in index_value_pairs[:n]:
        top_indices.append(pair[0])
    return top_indices


def tabulate_validation_scores(scores, keep=None):
    if not keep:
        keys = list(scores[0].keys())
    else:
        keys = keep
    headers = ["Rank"] + keys
    cols = [[i] for i in range(1, int(len(scores[0][list(scores[0].keys())[0]]) / 2) + 1)] + [[i] for i in range(-1, -int(len(scores[0][list(scores[0].keys())[0]]) / 2) - 1, -1)]
    for score in scores:
        for key in keys:
            for i, x in enumerate(score[key]):
                cols[i].append(x)

    print(tabulate(cols, headers=headers))
    pass


def get_score_change_pr_file(baseline, comparison_score):
    baseline_pr_file = {baseline[list(baseline.keys())[0]][i][0]: {} for i in
                        range(len(baseline[list(baseline.keys())[0]]))}
    for key in baseline:
        for file, point in baseline[key]:
            baseline_pr_file[file][key] = point
    change_pr_file = {baseline[list(baseline.keys())[0]][i][0]: {} for i in
                      range(len(baseline[list(baseline.keys())[0]]))}
    for key in comparison_score:
        for file, point in comparison_score[key]:
            change_pr_file[file][key] = point - baseline_pr_file[file][key]
    return change_pr_file


def get_files_with_n_extreme_changes(baseline, comparison_score, n):
    score_change_pr_file = get_score_change_pr_file(baseline, comparison_score)
    measures = list(list(score_change_pr_file.values())[0].keys())
    top_changes = {m: [] for m in measures}
    for measure in measures:
        file_with_measure_changes = [(key, score_change_pr_file[key][measure]) for key in score_change_pr_file]
        file_with_measure_changes.sort(key=lambda x: x[1])
        top_changes[measure] = file_with_measure_changes[:n] + file_with_measure_changes[-n:]
        top_changes[measure].sort(key=lambda x: x[1])
    return top_changes


def get_number_of_changes(baseline, comparison_score):
    score_change_pr_file = get_score_change_pr_file(baseline, comparison_score)
    measures = list(list(score_change_pr_file.values())[0].keys())
    change_counts = {m: [0, 0] for m in measures}
    for measure in measures:
        for file in score_change_pr_file.keys():
            if score_change_pr_file[file][measure] < 0:
                change_counts[measure][0] += 1
            else:
                change_counts[measure][1] += 1
    return change_counts


def tabulate_experiment_validations(baseline, other, val_folder, names=None, limit=None, keep=None):
    validation_files = [get_validation_files_by_score(p, val_folder, lim=limit) for p in [baseline] + other]
    if not names:
        names = [i.split("/")[1].replace(".pickle", "") for i in [baseline] + other]
    for i, vf in enumerate(validation_files):
        print(f"-------------------- {names[i]} --------------------")
        tabulate_validation_scores([vf], keep=keep)


def tabulate_experiment_changes(baseline, other, val_folder, names=None, limit=None, keep=None):
    baseline = get_validation_files_by_score(baseline, val_folder)
    changes = [(get_files_with_n_extreme_changes(baseline, get_validation_files_by_score(i, val_folder), limit), get_number_of_changes(baseline, get_validation_files_by_score(i, val_folder))) for i in other]
    if not names:
        names = [i.split("/")[1].replace(".pickle", "") for i in other]
    for i, c in enumerate(changes):
        print(f"------------------- {names[i]} --------------------")
        tabulate_validation_scores([c[0]], keep=keep)
        print(c[1])



if __name__ == "__main__":
    create_score_graphs(["pickles/guitarset_baseline.pickle", "pickles/guitarset_e1_rc20.pickle", "pickles/guitarset_e1_serumfx.pickle", "pickles/guitarset_e2_kontakt.pickle", "pickles/guitarset_e2_serum.pickle"], "Datasets/GuitarSet/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"] , header="GuitarSet")
    """
    print("GuitarSet Validation")
    tabulate_experiment_validations("pickles/guitarset_baseline.pickle", ["pickles/guitarset_e1_rc20.pickle", "pickles/guitarset_e1_serumfx.pickle", "pickles/guitarset_e2_kontakt.pickle", "pickles/guitarset_e2_serum.pickle"], "Datasets/GuitarSet/validation",
     ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])

    print("Guitarset Validation Changes")
    tabulate_experiment_changes("pickles/guitarset_baseline.pickle", ["pickles/guitarset_e1_rc20.pickle", "pickles/guitarset_e1_serumfx.pickle", "pickles/guitarset_e2_kontakt.pickle", "pickles/guitarset_e2_serum.pickle"], 
    "Datasets/GuitarSet/validation", names=["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], limit=3, keep=["Onset F1", "Frame F1", "Onset + offset F1"])
    """
    create_score_graphs(["pickles/urmp_baseline.pickle", "pickles/urmp_e1_rc20.pickle", "pickles/urmp_e1_serumfx.pickle", "pickles/urmp_e2_kontakt.pickle", "pickles/urmp_e2_serum.pickle"], "Datasets/URMP/Dataset/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"],  header="URMP" )
    """
    print("URMP Validation")
    tabulate_experiment_validations("pickles/urmp_baseline.pickle", ["pickles/urmp_e1_rc20.pickle", "pickles/urmp_e1_serumfx.pickle", "pickles/urmp_e2_kontakt.pickle", "pickles/urmp_e2_serum.pickle"], "Datasets/URMP/Dataset/validation",
    ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])

    print("URMP Validation Changes")
    tabulate_experiment_changes("pickles/urmp_baseline.pickle", ["pickles/urmp_e1_rc20.pickle", "pickles/urmp_e1_serumfx.pickle", "pickles/urmp_e2_kontakt.pickle", "pickles/urmp_e2_serum.pickle"],  "Datasets/URMP/Dataset/validation",
    names=["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], limit=3, keep=["Onset F1", "Frame F1", "Onset + offset F1"])
    """
    create_score_graphs(["pickles/guitarset_baseline.pickle", "pickles/guitarset_e1_rc20_13p.pickle", "pickles/guitarset_e1_rc20_25p.pickle", "pickles/guitarset_e1_rc20_50p.pickle", "pickles/guitarset_e1_rc20_75p.pickle", "pickles/guitarset_e1_rc20.pickle"], "Datasets/GuitarSet/validation", ["E0", "12.5%", "25%", "50%", "75%", "100%"],  header="GuitarSet Experiment 7")
    """
    print("Experiment 6 Validation")
    tabulate_experiment_validations("pickles/guitarset_baseline.pickle", ["pickles/guitarset_e1_rc20_13p.pickle", "pickles/guitarset_e1_rc20_25p.pickle", "pickles/guitarset_e1_rc20_50p.pickle", "pickles/guitarset_e1_rc20_75p.pickle", "pickles/guitarset_e1_rc20.pickle"], "Datasets/GuitarSet/validation", ["E0", "12.5%", "25%", "50%", "75%", "100%"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])
    print("Experiment 6 Validation Changes")
    tabulate_experiment_changes("pickles/guitarset_baseline.pickle", ["pickles/guitarset_e1_rc20_13p.pickle", "pickles/guitarset_e1_rc20_25p.pickle", "pickles/guitarset_e1_rc20_50p.pickle", "pickles/guitarset_e1_rc20_75p.pickle", "pickles/guitarset_e1_rc20.pickle"], "Datasets/GuitarSet/validation", ["E0", "12.5%", "25%", "50%", "75%", "100%"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])

    """
    create_score_graphs(["pickles/musicnet_baseline.pickle", "pickles/musicnet_e1_rc20.pickle", "pickles/musicnet_e1_serumfx.pickle", "pickles/musicnet_e2_kontakt.pickle", "pickles/musicnet_e2_serum.pickle"], "Datasets/musicnet/tracks/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"],  header="MusicNet")
    """
    print("MusicNet Validation")
    tabulate_experiment_validations("pickles/musicnet_baseline.pickle", ["pickles/musicnet_e1_rc20.pickle", "pickles/musicnet_e1_serumfx.pickle", "pickles/musicnet_e2_kontakt.pickle", "pickles/musicnet_e2_serum.pickle"], "Datasets/musicnet/tracks/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])
    print("Musicnet Validation Changes")
    tabulate_experiment_changes("pickles/musicnet_baseline.pickle", ["pickles/musicnet_e1_rc20.pickle", "pickles/musicnet_e1_serumfx.pickle", "pickles/musicnet_e2_kontakt.pickle", "pickles/musicnet_e2_serum.pickle"], "Datasets/musicnet/tracks/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])

    """
    create_score_graphs(["pickles/maestro_baseline.pickle", "pickles/maestro_e1_rc20.pickle",  "pickles/maestro_e1_serumfx.pickle",  "pickles/maestro_e2_kontakt.pickle",  "pickles/maestro_e2_serum.pickle"], "Datasets/maestro-v3.0.0/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"],  header="Maestro")
    """
    print("Maestro Validation")
    tabulate_experiment_validations("pickles/maestro_baseline.pickle", ["pickles/maestro_e1_rc20.pickle",  "pickles/maestro_e1_serumfx.pickle",  "pickles/maestro_e2_kontakt.pickle",  "pickles/maestro_e2_serum.pickle"], "Datasets/maestro-v3.0.0/validation", ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3, ["Onset F1", "Frame F1", "Onset + offset F1"])
    print("Maestro Validation Changes")
    tabulate_experiment_changes("pickles/maestro_baseline.pickle",
                                    ["pickles/maestro_e1_rc20.pickle", "pickles/maestro_e1_serumfx.pickle",
                                     "pickles/maestro_e2_kontakt.pickle", "pickles/maestro_e2_serum.pickle"],
                                    "Datasets/maestro-v3.0.0/validation",
                                    ["E0", "E1 - RC20", "E1 - SerumFX", "E2 - Kontakt", "E2 - Serum"], 3,
                                    ["Onset F1", "Frame F1", "Onset + offset F1"])f
                                    
    """