"""
A small visualization script to print Raw audio, FFT spectrogram, and Mel spectrogram
"""

import librosa
import numpy as np
import librosa.display
import matplotlib.pyplot as plt
y, sr = librosa.load("EFGABCDEF.wav")

# RAW
plt.figure(figsize=(20, 5))
librosa.display.waveplot(y, sr=sr)
plt.show()

# FFT
D = librosa.amplitude_to_db(np.abs(librosa.stft(y, n_fft=2048, hop_length=128)), ref=np.max)
plt.figure(figsize=(20, 5))
librosa.display.specshow(D, y_axis='log', x_axis='time', fmax=8000, fmin=0)
plt.title("FFT Spectrogram")
plt.show()

# Mel spectrogram
S = librosa.feature.melspectrogram(y=y, sr=16000, n_mels=128, n_fft=2048, hop_length=128)
S_dB = librosa.power_to_db(S, ref=np.max)
plt.figure(figsize=(20, 5))
img = librosa.display.specshow(S_dB, x_axis='time',
                         y_axis='mel', sr=sr, fmin=0)
plt.ylabel('Mels')
plt.title("Mel Spectrogram")
plt.show()
